@@ -36,18 +36,19 @@
 
 #include <QApplication>
 #include <QDBusConnection>
-#include <QDBusInterface>
 #include <QDBusPendingReply>
-#include <QDesktopWidget>
+#include <QDBusReply>
 #include <QMenu>
 #include <QPainter>
-#include <QPlatformSurfaceEvent>
 #include <QScreen>
 #include <QWhatsThis>
-#include <QWindow>
 
 #if HAVE_X11
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #include <QX11Info>
+#else
+#include <private/qtx11extras_p.h>
+#endif
 
 #include <X11/Xlib.h>
 #include <fixx11h.h>
@@ -115,11 +116,13 @@ MainWindow::MainWindow(QWidget *parent)
     connect(m_sessionStack, SIGNAL(sessionRemoved(int)), m_tabBar, SLOT(removeTab(int)));
     connect(m_sessionStack, SIGNAL(activeTitleChanged(QString)), m_titleBar, SLOT(setTitle(QString)));
     connect(m_sessionStack, SIGNAL(activeTitleChanged(QString)), this, SLOT(setWindowTitle(QString)));
+    connect(m_sessionStack, &SessionStack::wantsBlurChanged, this, &MainWindow::applyWindowProperties);
 
     connect(&m_mousePoller, SIGNAL(timeout()), this, SLOT(pollMouse()));
 
     connect(KWindowSystem::self(), SIGNAL(workAreaChanged()), this, SLOT(applyWindowGeometry()));
-    connect(QApplication::desktop(), SIGNAL(screenCountChanged(int)), this, SLOT(updateScreenMenu()));
+    connect(qApp, &QGuiApplication::screenAdded, this, &MainWindow::updateScreenMenu);
+    connect(qApp, &QGuiApplication::screenRemoved, this, &MainWindow::updateScreenMenu);
 
     applySettings();
 
@@ -226,12 +229,12 @@ void MainWindow::setupActions()
 
     KToggleFullScreenAction *fullScreenAction = new KToggleFullScreenAction(this);
     fullScreenAction->setWindow(this);
-    actionCollection()->setDefaultShortcut(fullScreenAction, QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_F11));
+    actionCollection()->setDefaultShortcut(fullScreenAction, QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_F11));
     m_actionCollection->addAction(QStringLiteral("view-full-screen"), fullScreenAction);
     connect(fullScreenAction, SIGNAL(toggled(bool)), this, SLOT(setFullScreen(bool)));
 
     QAction *action = KStandardAction::quit(this, SLOT(close()), actionCollection());
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_Q));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_Q));
     action = KStandardAction::aboutApp(m_helpMenu, SLOT(aboutApplication()), actionCollection());
     action = KStandardAction::reportBug(m_helpMenu, SLOT(reportBug()), actionCollection());
     action = KStandardAction::aboutKDE(m_helpMenu, SLOT(aboutKDE()), actionCollection());
@@ -272,28 +275,28 @@ void MainWindow::setupActions()
 
     action = actionCollection()->addAction(QStringLiteral("increase-window-width"));
     action->setText(xi18nc("@action", "Increase Window Width"));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::ALT + Qt::SHIFT + Qt::Key_Right));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::ALT | Qt::SHIFT | Qt::Key_Right));
     connect(action, SIGNAL(triggered()), this, SLOT(increaseWindowWidth()));
 
     action = actionCollection()->addAction(QStringLiteral("decrease-window-width"));
     action->setText(xi18nc("@action", "Decrease Window Width"));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::ALT + Qt::SHIFT + Qt::Key_Left));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::ALT | Qt::SHIFT | Qt::Key_Left));
     connect(action, SIGNAL(triggered()), this, SLOT(decreaseWindowWidth()));
 
     action = actionCollection()->addAction(QStringLiteral("increase-window-height"));
     action->setText(xi18nc("@action", "Increase Window Height"));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::ALT + Qt::SHIFT + Qt::Key_Down));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::ALT | Qt::SHIFT | Qt::Key_Down));
     connect(action, SIGNAL(triggered()), this, SLOT(increaseWindowHeight()));
 
     action = actionCollection()->addAction(QStringLiteral("decrease-window-height"));
     action->setText(xi18nc("@action", "Decrease Window Height"));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::ALT + Qt::SHIFT + Qt::Key_Up));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::ALT | Qt::SHIFT | Qt::Key_Up));
     connect(action, SIGNAL(triggered()), this, SLOT(decreaseWindowHeight()));
 
     action = actionCollection()->addAction(QStringLiteral("new-session"));
     action->setText(xi18nc("@action", "New Session"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("tab-new")));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_T));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_T));
     connect(action, SIGNAL(triggered()), m_sessionStack, SLOT(addSession()));
 
     action = actionCollection()->addAction(QStringLiteral("new-session-two-horizontal"));
@@ -314,7 +317,7 @@ void MainWindow::setupActions()
     action = actionCollection()->addAction(QStringLiteral("close-session"));
     action->setText(xi18nc("@action", "Close Session"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("tab-close")));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_W));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_W));
     connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
     m_contextDependentActions << action;
 
@@ -333,49 +336,49 @@ void MainWindow::setupActions()
     action = actionCollection()->addAction(QStringLiteral("move-session-left"));
     action->setText(xi18nc("@action", "Move Session Left"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("arrow-left")));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_Left));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_Left));
     connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
     m_contextDependentActions << action;
 
     action = actionCollection()->addAction(QStringLiteral("move-session-right"));
     action->setText(xi18nc("@action", "Move Session Right"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("arrow-right")));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_Right));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_Right));
     connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
     m_contextDependentActions << action;
 
     action = actionCollection()->addAction(QStringLiteral("grow-terminal-right"));
     action->setText(xi18nc("@action", "Grow Terminal to the Right"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("arrow-right")));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL + Qt::ALT + Qt::Key_Right));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::ALT | Qt::Key_Right));
     connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
     m_contextDependentActions << action;
 
     action = actionCollection()->addAction(QStringLiteral("grow-terminal-left"));
     action->setText(xi18nc("@action", "Grow Terminal to the Left"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("arrow-left")));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL + Qt::ALT + Qt::Key_Left));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::ALT | Qt::Key_Left));
     connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
     m_contextDependentActions << action;
 
     action = actionCollection()->addAction(QStringLiteral("grow-terminal-top"));
     action->setText(xi18nc("@action", "Grow Terminal to the Top"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("arrow-up")));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL + Qt::ALT + Qt::Key_Up));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::ALT | Qt::Key_Up));
     connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
     m_contextDependentActions << action;
 
     action = actionCollection()->addAction(QStringLiteral("grow-terminal-bottom"));
     action->setText(xi18nc("@action", "Grow Terminal to the Bottom"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("arrow-down")));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL + Qt::ALT + Qt::Key_Down));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::ALT | Qt::Key_Down));
     connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
     m_contextDependentActions << action;
 
     action = actionCollection()->addAction(QStringLiteral("rename-session"));
     action->setText(xi18nc("@action", "Rename Session..."));
     action->setIcon(QIcon::fromTheme(QStringLiteral("edit-rename")));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL + Qt::ALT + Qt::Key_S));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::ALT | Qt::Key_S));
     connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
     m_contextDependentActions << action;
 
@@ -394,21 +397,21 @@ void MainWindow::setupActions()
     action = actionCollection()->addAction(QStringLiteral("close-active-terminal"));
     action->setText(xi18nc("@action", "Close Active Terminal"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("view-close")));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_R));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_R));
     connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
     m_contextDependentActions << action;
 
     action = actionCollection()->addAction(QStringLiteral("split-left-right"));
     action->setText(xi18nc("@action", "Split Left/Right"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("view-split-left-right")));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL + Qt::Key_ParenLeft));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::Key_ParenLeft));
     connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
     m_contextDependentActions << action;
 
     action = actionCollection()->addAction(QStringLiteral("split-top-bottom"));
     action->setText(xi18nc("@action", "Split Top/Bottom"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("view-split-top-bottom")));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL + Qt::Key_ParenRight));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::Key_ParenRight));
     connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
     m_contextDependentActions << action;
 
@@ -426,21 +429,21 @@ void MainWindow::setupActions()
 
     action = actionCollection()->addAction(QStringLiteral("toggle-session-monitor-activity"));
     action->setText(xi18nc("@action", "Monitor for Activity"));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_A));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_A));
     action->setCheckable(true);
     connect(action, SIGNAL(triggered(bool)), this, SLOT(handleContextDependentToggleAction(bool)));
     m_contextDependentActions << action;
 
     action = actionCollection()->addAction(QStringLiteral("toggle-session-monitor-silence"));
     action->setText(xi18nc("@action", "Monitor for Silence"));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_I));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_I));
     action->setCheckable(true);
     connect(action, SIGNAL(triggered(bool)), this, SLOT(handleContextDependentToggleAction(bool)));
     m_contextDependentActions << action;
 
     action = actionCollection()->addAction(QStringLiteral("toggle-titlebar"));
     action->setText(xi18nc("@action", "Toggle Titlebar"));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_M));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_M));
     action->setCheckable(true);
     connect(action, SIGNAL(triggered()), this, SLOT(handleToggleTitlebar()));
 
@@ -455,7 +458,7 @@ void MainWindow::setupActions()
             actionCollection()->setDefaultShortcut(action, QStringLiteral("Alt+%1").arg(i));
         } else {
             // add default shortcut bindings for the 10th session
-            actionCollection()->setDefaultShortcut(action, Qt::ALT + Qt::Key_0);
+            actionCollection()->setDefaultShortcut(action, Qt::ALT | Qt::Key_0);
         }
     }
 
@@ -645,8 +648,8 @@ void MainWindow::handleToggleTitlebar()
             "You are about to hide the title bar. This will keep you "
             "from accessing the settings menu via the mouse. To show "
             "the title bar again press the keyboard shortcut (default "
-            "Ctrl+Shift+m) or access the settings menu via keyborad "
-            "shortcut (defult: Ctrl+Shift+,).";
+            "Ctrl+Shift+m) or access the settings menu via keyboard "
+            "shortcut (default: Ctrl+Shift+,).";
 
         const int result = KMessageBox::warningContinueCancel(this,
                                                               xi18nc("@info", message),
@@ -929,7 +932,7 @@ void MainWindow::applyWindowProperties()
         KWindowSystem::setState(winId(), NET::KeepAbove | NET::Sticky | NET::SkipTaskbar | NET::SkipPager);
 
     KWindowSystem::setOnAllDesktops(winId(), Settings::showOnAllDesktops());
-    KWindowEffects::enableBlurBehind(windowHandle(), Settings::blur());
+    KWindowEffects::enableBlurBehind(windowHandle(), m_sessionStack->wantsBlur());
 }
 
 void MainWindow::applyWindowGeometry()
@@ -1090,10 +1093,13 @@ void MainWindow::paintEvent(QPaintEvent *event)
 
 void MainWindow::moveEvent(QMoveEvent *event)
 {
-    int widgetCurrentScreen = QApplication::desktop()->screenNumber(this);
+    const QList<QScreen *> screens = qApp->screens();
+    const QScreen *widgetScreen = QGuiApplication::screenAt(pos());
+    auto it = std::find(screens.begin(), screens.end(), widgetScreen);
+    const int currentScreenNumber = std::distance(screens.begin(), it);
 
-    if (Settings::screen() && widgetCurrentScreen != -1 && widgetCurrentScreen != getScreen()) {
-        Settings::setScreen(widgetCurrentScreen + 1);
+    if (Settings::screen() && currentScreenNumber != -1 && currentScreenNumber != getScreen()) {
+        Settings::setScreen(currentScreenNumber + 1);
 
         updateScreenMenu();
 
@@ -1487,6 +1493,19 @@ void MainWindow::setFullScreen(bool state)
 int MainWindow::getScreen()
 {
     if (!Settings::screen() || Settings::screen() > QGuiApplication::screens().length()) {
+        auto message = QDBusMessage::createMethodCall(QStringLiteral("org.kde.KWin"),
+                                                      QStringLiteral("/KWin"),
+                                                      QStringLiteral("org.kde.KWin"),
+                                                      QStringLiteral("activeOutputName"));
+        QDBusReply<QString> reply = QDBusConnection::sessionBus().call(message);
+
+        if (reply.isValid()) {
+            const auto screens = QGuiApplication::screens();
+            for (int i = 0; i < screens.size(); ++i) {
+                if (screens[i]->name() == reply.value())
+                    return i;
+            }
+        }
         // Right after unplugging an external monitor and the Yakuake window was on
         // that monitor, QGuiApplication::screenAt() can return nullptr so we fallback on
         // the first monitor.
